@@ -25,10 +25,10 @@ class CityScapesDataset(BaseDataset):
         (0, 80, 100), (0, 0, 230), (119, 11, 32)
     ]
     clsid2label = {
-        -1: 255, 0: 255, 1: 255, 2: 255, 3: 255, 4: 255, 5: 255, 6: 255,
-        7: 0, 8: 1, 9: 255, 10: 255, 11: 2, 12: 3, 13: 4, 14: 255, 
-        15: 255, 16: 255, 17: 5, 18: 255, 19: 6, 20: 7, 21: 8, 22: 9, 
-        23: 10, 24: 11, 25: 12, 26: 13, 27: 14, 28: 15, 29: 255, 30: 255, 
+        -1: -100, 0: -100, 1: -100, 2: -100, 3: -100, 4: -100, 5: -100, 6: -100,
+        7: 0, 8: 1, 9: -100, 10: -100, 11: 2, 12: 3, 13: 4, 14: -100, 
+        15: -100, 16: -100, 17: 5, 18: -100, 19: 6, 20: 7, 21: 8, 22: 9, 
+        23: 10, 24: 11, 25: 12, 26: 13, 27: 14, 28: 15, 29: -100, 30: -100, 
         31: 16, 32: 17, 33: 18
     }
     assert num_classes == len(classnames) and num_classes == len(palette)
