@@ -103,7 +103,7 @@ class COCOStuff10kDataset(BaseDataset):
         'wall-stone', 'wall-tile', 'wall-wood', 'water-other', 'waterdrops', 'window-blind', 'window-other', 'wood'
     ]
     palette = BaseDataset.randompalette(num_classes)
-    clsid2label = {0: 255}
+    clsid2label = {0: -100, 255: -100}
     for i in range(1, num_classes+1): clsid2label[i] = i - 1
     assert num_classes == len(classnames) and num_classes == len(palette)
     def __init__(self, mode, logger_handle, dataset_cfg):
@@ -144,7 +144,7 @@ class COCOStuffDataset(BaseDataset):
         'wall-stone', 'wall-tile', 'wall-wood', 'water-other', 'waterdrops', 'window-blind', 'window-other', 'wood'
     ]
     palette = BaseDataset.randompalette(num_classes)
-    clsid2label = {0: 255}
+    clsid2label = {0: -100, 255: -100}
     for i in range(1, num_classes+1): clsid2label[i] = i - 1
     assert num_classes == len(classnames) and num_classes == len(palette)
     def __init__(self, mode, logger_handle, dataset_cfg):
