@@ -1,5 +1,6 @@
 '''initialize'''
 from .ffn import FFN
 from .mha import MultiheadAttention
+from .pe import PositionEmbeddingSine
 from .embed import PatchEmbed, PatchMerging, AdaptivePadding
-from .misc import nchwtonlc, nlctonchw, nlc2nchw2nlc, nchw2nlc2nchw
\ No newline at end of file
+from .shape import nchwtonlc, nlctonchw, nlc2nchw2nlc, nchw2nlc2nchw
\ No newline at end of file
