@@ -8,7 +8,9 @@ import os
 import torch
 import torch.nn as nn
 import torch.utils.model_zoo as model_zoo
+import torch.utils.checkpoint as checkpoint
 from functools import partial
+from .bricks.normalization.grn import GRN
 from .bricks.dropout.droppath import DropPath
 from .bricks import BuildNormalization, BuildActivation
 from .bricks.normalization.layernorm2d import LayerNorm2d
@@ -29,9 +31,10 @@ AUTO_ASSERT_STRUCTURE_TYPES = {}
 
 '''ConvNeXtBlock'''
 class ConvNeXtBlock(nn.Module):
-    def __init__(self, in_channels, norm_cfg=None, act_cfg=None, mlp_ratio=4., linear_pw_conv=True, drop_path_rate=0., layer_scale_init_value=1e-6):
+    def __init__(self, in_channels, dw_conv_cfg=dict(kernel_size=7, padding=3), norm_cfg=None, act_cfg=None, mlp_ratio=4., linear_pw_conv=True, drop_path_rate=0., layer_scale_init_value=1e-6, use_grn=False, use_checkpoint=False):
         super(ConvNeXtBlock, self).__init__()
-        self.depthwise_conv = nn.Conv2d(in_channels, in_channels, kernel_size=7, padding=3, groups=in_channels)
+        self.use_checkpoint = use_checkpoint
+        self.depthwise_conv = nn.Conv2d(in_channels, in_channels, groups=in_channels, **dw_conv_cfg)
         self.linear_pw_conv = linear_pw_conv
         self.norm = BuildNormalization(placeholder=in_channels, norm_cfg=norm_cfg)
         mid_channels = int(mlp_ratio * in_channels)
@@ -42,40 +45,63 @@ class ConvNeXtBlock(nn.Module):
         self.pointwise_conv1 = pw_conv(in_channels, mid_channels)
         self.act = BuildActivation(act_cfg)
         self.pointwise_conv2 = pw_conv(mid_channels, in_channels)
+        if use_grn:
+            self.grn = GRN(mid_channels)
+        else:
+            self.grn = None
         self.gamma = nn.Parameter(
             layer_scale_init_value * torch.ones((in_channels)), requires_grad=True
         ) if layer_scale_init_value > 0 else None
         self.drop_path = DropPath(drop_path_rate) if drop_path_rate > 0. else nn.Identity()
     '''forward'''
-    def forward(self, x):
-        shortcut = x
-        x = self.depthwise_conv(x)
-        x = self.norm(x)
-        if self.linear_pw_conv:
-            x = x.permute(0, 2, 3, 1)
-        x = self.pointwise_conv1(x)
-        x = self.act(x)
-        x = self.pointwise_conv2(x)
-        if self.linear_pw_conv:
-            x = x.permute(0, 3, 1, 2)
-        if self.gamma is not None:
-            x = x.mul(self.gamma.view(1, -1, 1, 1))
-        x = shortcut + self.drop_path(x)
+    def forward(self, x: torch.Tensor):
+        def _forward(x: torch.Tensor):
+            shortcut = x
+            x = self.depthwise_conv(x)
+            if self.linear_pw_conv:
+                x = x.permute(0, 2, 3, 1)
+                x = self.norm(x, data_format='channel_last')
+                x = self.pointwise_conv1(x)
+                x = self.act(x)
+                if self.grn is not None:
+                    x = self.grn(x, data_format='channel_last')
+                x = self.pointwise_conv2(x)
+                x = x.permute(0, 3, 1, 2)
+            else:
+                x = self.norm(x, data_format='channel_first')
+                x = self.pointwise_conv1(x)
+                x = self.act(x)
+                if self.grn is not None:
+                    x = self.grn(x, data_format='channel_first')
+                x = self.pointwise_conv2(x)
+            if self.gamma is not None:
+                x = x.mul(self.gamma.view(1, -1, 1, 1))
+            x = shortcut + self.drop_path(x)
+            return x
+        if self.use_checkpoint and x.requires_grad:
+            x = checkpoint.checkpoint(_forward, x)
+        else:
+            x = _forward(x)
         return x
 
 
 '''ConvNeXt'''
 class ConvNeXt(nn.Module):
     arch_settings = {
+        'atto': {'depths': [2, 2, 6, 2], 'channels': [40, 80, 160, 320]},
+        'femto': {'depths': [2, 2, 6, 2], 'channels': [48, 96, 192, 384]},
+        'pico': {'depths': [2, 2, 6, 2], 'channels': [64, 128, 256, 512]},
+        'nano': {'depths': [2, 2, 8, 2], 'channels': [80, 160, 320, 640]},
         'tiny': {'depths': [3, 3, 9, 3], 'channels': [96, 192, 384, 768]},
         'small': {'depths': [3, 3, 27, 3], 'channels': [96, 192, 384, 768]},
         'base': {'depths': [3, 3, 27, 3], 'channels': [128, 256, 512, 1024]},
         'large': {'depths': [3, 3, 27, 3], 'channels': [192, 384, 768, 1536]},
         'xlarge': {'depths': [3, 3, 27, 3], 'channels': [256, 512, 1024, 2048]},
+        'huge': {'depths': [3, 3, 27, 3], 'channels': [352, 704, 1408, 2816]}
     }
     def __init__(self, structure_type, arch='tiny', in_channels=3, stem_patch_size=4, norm_cfg={'type': 'LayerNorm2d', 'eps': 1e-6}, act_cfg={'type': 'GELU'},
-                 linear_pw_conv=True, drop_path_rate=0., layer_scale_init_value=1e-6, out_indices=(0, 1, 2, 3), gap_before_final_norm=True,
-                 pretrained=True, pretrained_model_path=''):
+                 linear_pw_conv=True, use_grn=False, drop_path_rate=0., layer_scale_init_value=1e-6, out_indices=(0, 1, 2, 3), gap_before_final_norm=True,
+                 pretrained=True, pretrained_model_path='', use_checkpoint=False):
         super(ConvNeXt, self).__init__()
         # set attributes
         self.structure_type = structure_type
@@ -90,6 +116,8 @@ class ConvNeXt(nn.Module):
         self.gap_before_final_norm = gap_before_final_norm
         self.pretrained = pretrained
         self.pretrained_model_path = pretrained_model_path
+        self.use_checkpoint = use_checkpoint
+        self.use_grn = use_grn
         arch = self.arch_settings[arch]
         self.depths = arch['depths']
         self.channels = arch['channels']
@@ -109,10 +137,9 @@ class ConvNeXt(nn.Module):
         dpr = [x.item() for x in torch.linspace(0, drop_path_rate, sum(self.depths))]
         # 4 downsample layers between stages, including the stem layer.
         self.downsample_layers = nn.ModuleList()
-        norm_layer = BuildNormalization(placeholder=self.channels[0], norm_cfg=norm_cfg)
         stem = nn.Sequential(
             nn.Conv2d(in_channels, self.channels[0], kernel_size=stem_patch_size, stride=stem_patch_size),
-            norm_layer,
+            BuildNormalization(placeholder=self.channels[0], norm_cfg=norm_cfg),
         )
         self.downsample_layers.append(stem)
         # 4 feature resolution stages, each consisting of multiple residual blocks
@@ -123,14 +150,14 @@ class ConvNeXt(nn.Module):
             channels = self.channels[i]
             if i >= 1:
                 downsample_layer = nn.Sequential(
-                    LayerNorm2d(self.channels[i - 1]),
+                    BuildNormalization(placeholder=self.channels[i - 1], norm_cfg=norm_cfg),
                     nn.Conv2d(self.channels[i - 1], channels, kernel_size=2, stride=2),
                 )
                 self.downsample_layers.append(downsample_layer)
             stage = nn.Sequential(*[
                 ConvNeXtBlock(
-                    in_channels=channels, drop_path_rate=dpr[block_idx + j], norm_cfg=norm_cfg, act_cfg=act_cfg,
-                    linear_pw_conv=linear_pw_conv, layer_scale_init_value=layer_scale_init_value
+                    in_channels=channels, drop_path_rate=dpr[block_idx + j], norm_cfg=norm_cfg, act_cfg=act_cfg, linear_pw_conv=linear_pw_conv, 
+                    layer_scale_init_value=layer_scale_init_value, use_grn=use_grn, use_checkpoint=use_checkpoint
                 ) for j in range(depth)
             ])
             block_idx += depth
@@ -142,7 +169,7 @@ class ConvNeXt(nn.Module):
         if pretrained:
             self.loadpretrainedweights(structure_type, pretrained_model_path)
     '''forward'''
-    def forward(self, x):
+    def forward(self, x: torch.Tensor):
         outs = []
         for i, stage in enumerate(self.stages):
             x = self.downsample_layers[i](x)
