@@ -8,6 +8,7 @@ import math
 import torch
 import torch.nn as nn
 import torch.nn.functional as F
+import torch.utils.checkpoint as checkpoint
 from .resnet import ResNet
 from ...utils import loadpretrainedweights
 from .resnet import Bottleneck as _Bottleneck
@@ -31,7 +32,7 @@ class RSoftmax(nn.Module):
         self.radix = radix
         self.groups = groups
     '''forward'''
-    def forward(self, x):
+    def forward(self, x: torch.Tensor):
         batch = x.size(0)
         if self.radix > 1:
             x = x.view(batch, self.groups, self.radix, -1).transpose(1, 2)
@@ -56,7 +57,7 @@ class SplitAttentionConv2d(nn.Module):
         self.fc2 = nn.Conv2d(inter_channels, channels * radix, kernel_size=1, stride=1, padding=0, groups=groups)
         self.rsoftmax = RSoftmax(radix, groups)
     '''forward'''
-    def forward(self, x):
+    def forward(self, x: torch.Tensor):
         x = self.conv(x)
         x = self.bn0(x)
         x = self.relu(x)
@@ -85,36 +86,44 @@ class SplitAttentionConv2d(nn.Module):
 class Bottleneck(_Bottleneck):
     expansion = 4
     def __init__(self, inplanes, planes, groups=1, base_width=4, base_channels=64, radix=2, reduction_factor=4, use_avg_after_block_conv2=True,
-                 stride=1, dilation=1, downsample=None, norm_cfg=None, act_cfg=None):
-        super(Bottleneck, self).__init__(inplanes, planes, stride, dilation, downsample, norm_cfg, act_cfg)
+                 stride=1, dilation=1, downsample=None, norm_cfg=None, act_cfg=None, style='pytorch', use_checkpoint=False):
+        super(Bottleneck, self).__init__(
+            inplanes=inplanes, planes=planes, stride=stride, dilation=dilation, downsample=downsample, norm_cfg=norm_cfg, act_cfg=act_cfg, style=style, use_checkpoint=use_checkpoint,
+        )
         if groups == 1: width = planes
         else: width = math.floor(planes * (base_width / base_channels)) * groups
-        self.use_avg_after_block_conv2 = use_avg_after_block_conv2 and self.stride > 1
-        self.conv1 = nn.Conv2d(inplanes, width, kernel_size=1, stride=1, padding=0, bias=False)
+        self.use_avg_after_block_conv2 = use_avg_after_block_conv2 and self.conv2_stride > 1
+        self.conv1 = nn.Conv2d(inplanes, width, kernel_size=1, stride=self.conv1_stride, padding=0, bias=False)
         self.bn1 = BuildNormalization(placeholder=width, norm_cfg=norm_cfg)
         self.conv2 = SplitAttentionConv2d(
-            in_channels=width, channels=width, kernel_size=3, stride=1 if self.use_avg_after_block_conv2 else self.stride, padding=dilation,
+            in_channels=width, channels=width, kernel_size=3, stride=1 if self.use_avg_after_block_conv2 else self.conv2_stride, padding=dilation,
             dilation=dilation, groups=groups, radix=radix, reduction_factor=reduction_factor, norm_cfg=norm_cfg, act_cfg=act_cfg,
         )
         delattr(self, 'bn2')
         self.conv3 = nn.Conv2d(width, planes * self.expansion, kernel_size=1, stride=1, padding=0, bias=False)
         self.bn3 = BuildNormalization(placeholder=planes * self.expansion, norm_cfg=norm_cfg)
         if self.use_avg_after_block_conv2: 
-            self.avg_layer = nn.AvgPool2d(3, self.stride, padding=1)
+            self.avg_layer = nn.AvgPool2d(3, self.conv2_stride, padding=1)
     '''forward'''
-    def forward(self, x):
-        identity = x
-        out = self.conv1(x)
-        out = self.bn1(out)
-        out = self.relu(out)
-        out = self.conv2(out)
-        if self.use_avg_after_block_conv2: 
-            out = self.avg_layer(out)
-        out = self.conv3(out)
-        out = self.bn3(out)
-        if self.downsample is not None: 
-            identity = self.downsample(x)
-        out += identity
+    def forward(self, x: torch.Tensor):
+        def _forward(x):
+            identity = x
+            out = self.conv1(x)
+            out = self.bn1(out)
+            out = self.relu(out)
+            out = self.conv2(out)
+            if self.use_avg_after_block_conv2: 
+                out = self.avg_layer(out)
+            out = self.conv3(out)
+            out = self.bn3(out)
+            if self.downsample is not None: 
+                identity = self.downsample(x)
+            out += identity
+            return out
+        if self.use_checkpoint and x.requires_grad:
+            out = checkpoint.checkpoint(_forward, x)
+        else:
+            out = _forward(x)
         out = self.relu(out)
         return out
 
@@ -127,15 +136,19 @@ class ResNeSt(ResNet):
         152: (Bottleneck, (3, 8, 36, 3)),
         200: (Bottleneck, (3, 24, 36, 3))
     }
-    def __init__(self, structure_type, groups=1, base_width=4, radix=2, reduction_factor=4, use_avg_after_block_conv2=True,
-                 in_channels=3, base_channels=64, stem_channels=128, depth=101, outstride=8, contract_dilation=True, use_conv3x3_stem=True, 
+    def __init__(self, structure_type, groups=1, base_width=4, radix=2, reduction_factor=4, use_avg_after_block_conv2=True, use_checkpoint=False,
+                 in_channels=3, base_channels=64, stem_channels=128, depth=101, outstride=8, contract_dilation=True, use_conv3x3_stem=True, style='pytorch',
                  out_indices=(0, 1, 2, 3), use_avg_for_downsample=True, norm_cfg={'type': 'SyncBatchNorm'}, act_cfg={'type': 'ReLU', 'inplace': True}, 
                  pretrained=True, pretrained_model_path=''):
-        self.extra_args_for_makelayer = {
+        self.block_extra_args = {
             'radix': radix, 'groups': groups, 'base_width': base_width, 'reduction_factor': reduction_factor, 'base_channels': base_channels, 
             'use_avg_after_block_conv2': use_avg_after_block_conv2,
         }
-        super(ResNeSt, self).__init__(structure_type, in_channels, base_channels, stem_channels, depth, outstride, contract_dilation, use_conv3x3_stem, out_indices, use_avg_for_downsample, norm_cfg, act_cfg, False, '')
+        super(ResNeSt, self).__init__(
+            structure_type=structure_type, in_channels=in_channels, base_channels=base_channels, stem_channels=stem_channels, depth=depth, outstride=outstride, contract_dilation=contract_dilation, 
+            use_conv3x3_stem=use_conv3x3_stem, out_indices=out_indices, use_avg_for_downsample=use_avg_for_downsample, norm_cfg=norm_cfg, act_cfg=act_cfg, style=style, use_checkpoint=use_checkpoint,
+            pretrained=False, pretrained_model_path=''
+        )
         # set attributes
         self.structure_type = structure_type
         self.groups = groups
@@ -156,6 +169,8 @@ class ResNeSt(ResNet):
         self.act_cfg = act_cfg
         self.pretrained = pretrained
         self.pretrained_model_path = pretrained_model_path
+        self.style = style
+        self.use_checkpoint = use_checkpoint
         # assert
         if structure_type in AUTO_ASSERT_STRUCTURE_TYPES:
             for key, value in AUTO_ASSERT_STRUCTURE_TYPES[structure_type].items():
@@ -167,25 +182,8 @@ class ResNeSt(ResNet):
             )
             self.load_state_dict(state_dict, strict=False)
     '''makelayer'''
-    def makelayer(self, block, inplanes, planes, num_blocks, stride=1, dilation=1, contract_dilation=True, use_avg_for_downsample=False, norm_cfg=None, act_cfg=None):
-        downsample = None
-        dilations = [dilation] * num_blocks
-        if contract_dilation and dilation > 1: dilations[0] = dilation // 2
-        if stride != 1 or inplanes != planes * block.expansion:
-            if use_avg_for_downsample:
-                downsample = nn.Sequential(
-                    nn.AvgPool2d(kernel_size=stride, stride=stride, ceil_mode=True, count_include_pad=False),
-                    nn.Conv2d(inplanes, planes * block.expansion, kernel_size=1, stride=1, padding=0, bias=False),
-                    BuildNormalization(placeholder=planes * block.expansion, norm_cfg=norm_cfg)
-                )
-            else:
-                downsample = nn.Sequential(
-                    nn.Conv2d(inplanes, planes * block.expansion, kernel_size=1, stride=stride, padding=0, bias=False),
-                    BuildNormalization(placeholder=planes * block.expansion, norm_cfg=norm_cfg)
-                )
-        layers = []
-        layers.append(block(inplanes, planes, stride=stride, dilation=dilations[0], downsample=downsample, norm_cfg=norm_cfg, act_cfg=act_cfg, **self.extra_args_for_makelayer))
-        self.inplanes = planes * block.expansion
-        for i in range(1, num_blocks): 
-            layers.append(block(planes * block.expansion, planes, stride=1, dilation=dilations[i], norm_cfg=norm_cfg, act_cfg=act_cfg, **self.extra_args_for_makelayer))
-        return nn.Sequential(*layers)
\ No newline at end of file
+    def makelayer(self, block, inplanes, planes, num_blocks, stride=1, dilation=1, contract_dilation=True, use_avg_for_downsample=False, norm_cfg=None, act_cfg=None, style='pytorch', use_checkpoint=False):
+        return super().makelayer(
+            block=block, inplanes=inplanes, planes=planes, num_blocks=num_blocks, stride=stride, dilation=dilation, contract_dilation=contract_dilation, use_avg_for_downsample=use_avg_for_downsample, 
+            norm_cfg=norm_cfg, act_cfg=act_cfg, style=style, use_checkpoint=use_checkpoint, block_extra_args=self.block_extra_args
+        )
\ No newline at end of file
