@@ -12,7 +12,7 @@ import torch.nn.functional as F
 import torch.utils.model_zoo as model_zoo
 import torch.utils.checkpoint as checkpoint
 from collections import OrderedDict
-from .bricks import BuildNormalization, BuildDropout, FFN, PatchEmbed, PatchMerging
+from .bricks import BuildNormalization, BuildDropout, FFN, PatchEmbed, PatchMerging, tolen2tuple
 
 
 '''DEFAULT_MODEL_URLS'''
@@ -24,6 +24,7 @@ DEFAULT_MODEL_URLS = {
     'swin_base_patch4_window12_384_22k': 'https://github.com/SwinTransformer/storage/releases/download/v1.0.0/swin_base_patch4_window12_384_22k.pth',
     'swin_base_patch4_window7_224_22k': 'https://github.com/SwinTransformer/storage/releases/download/v1.0.0/swin_base_patch4_window7_224_22k.pth',
     'swin_large_patch4_window12_384_22k': 'https://github.com/SwinTransformer/storage/releases/download/v1.0.0/swin_large_patch4_window12_384_22k.pth',
+    'swin_large_patch4_window12_384_22kto1k': 'https://github.com/SwinTransformer/storage/releases/download/v1.0.0/swin_large_patch4_window12_384_22kto1k.pth',
 }
 '''AUTO_ASSERT_STRUCTURE_TYPES'''
 AUTO_ASSERT_STRUCTURE_TYPES = {
@@ -62,6 +63,11 @@ AUTO_ASSERT_STRUCTURE_TYPES = {
         'depths': [2, 2, 18, 2], 'num_heads': [6, 12, 24, 48], 'qkv_bias': True, 'qk_scale': None, 'patch_norm': True,
         'drop_rate': 0., 'attn_drop_rate': 0., 'drop_path_rate': 0.3, 'use_abs_pos_embed': False,
     },
+    'swin_large_patch4_window12_384_22kto1k': {
+        'pretrain_img_size': 384, 'in_channels': 3, 'embed_dims': 192, 'patch_size': 4, 'window_size': 12, 'mlp_ratio': 4,
+        'depths': [2, 2, 18, 2], 'num_heads': [6, 12, 24, 48], 'qkv_bias': True, 'qk_scale': None, 'patch_norm': True,
+        'drop_rate': 0., 'attn_drop_rate': 0., 'drop_path_rate': 0.3, 'use_abs_pos_embed': False,
+    },
 }
 
 
@@ -122,7 +128,7 @@ class ShiftWindowMSA(nn.Module):
         self.shift_size = shift_size
         assert 0 <= self.shift_size < self.window_size
         self.w_msa = WindowMSA(
-            embed_dims=embed_dims, num_heads=num_heads, window_size=(window_size, window_size), qkv_bias=qkv_bias,
+            embed_dims=embed_dims, num_heads=num_heads, window_size=tolen2tuple(window_size), qkv_bias=qkv_bias,
             qk_scale=qk_scale, attn_drop_rate=attn_drop_rate, proj_drop_rate=proj_drop_rate,
         )
         self.drop = BuildDropout(dropout_cfg)
@@ -213,7 +219,7 @@ class SwinBlock(nn.Module):
             act_cfg=act_cfg, add_identity=True,
         )
     '''forward'''
-    def forward(self, x, hw_shape):
+    def forward(self, x: torch.Tensor, hw_shape):
         def _forward(x):
             identity = x
             x = self.norm1(x)
@@ -291,7 +297,7 @@ class SwinTransformer(nn.Module):
         if structure_type in AUTO_ASSERT_STRUCTURE_TYPES:
             for key, value in AUTO_ASSERT_STRUCTURE_TYPES[structure_type].items():
                 assert hasattr(self, key) and (getattr(self, key) == value)
-        if isinstance(pretrain_img_size, int): pretrain_img_size = (pretrain_img_size, pretrain_img_size)
+        pretrain_img_size = tolen2tuple(pretrain_img_size)
         self.pretrain_img_size = pretrain_img_size
         # patch embedding
         self.patch_embed = PatchEmbed(
