@@ -1,13 +1,14 @@
 '''
 Function:
-    Build distributed model
+    Implementation of BuildDistributedModel
 Author:
     Zhenchao Jin
 '''
-import torch
+import copy
 import torch.nn as nn
 
 
 '''BuildDistributedModel'''
 def BuildDistributedModel(model, model_cfg):
+    model_cfg = copy.deepcopy(model_cfg)
     return nn.parallel.DistributedDataParallel(model, **model_cfg)
\ No newline at end of file
